@@ -16,32 +16,54 @@ from uuid import uuid4
 
 from django.utils import timezone
 
-from adjutant.actions import user_store
+from confspirator import groups
+from confspirator import fields
+
+from adjutant.config import CONF
+from adjutant.common import user_store
+from adjutant.common.utils import str_datetime
+from adjutant.actions.utils import validate_steps
 from adjutant.actions.v1.base import (
     BaseAction, UserNameAction, UserMixin, ProjectMixin)
+from adjutant.actions.v1 import serializers
 
 
 class NewProjectAction(BaseAction, ProjectMixin, UserMixin):
     """
     Creates a new project for the current keystone_user.
 
-    This action can only be used for an autheticated taskview.
+    This action can only be used for an autheticated task.
     """
 
     required = [
         'domain_id',
         'parent_id',
         'project_name',
+        'description',
     ]
 
+    serializer = serializers.NewProjectSerializer
+
+    config_group = groups.DynamicNameConfigGroup(
+        children=[
+            fields.ListConfig(
+                "default_roles",
+                help_text="Roles to be given on project to the creating user.",
+                default=[],
+                sample_default=["member", "project_admin"]
+            ),
+        ],
+    )
+
     def __init__(self, *args, **kwargs):
         super(NewProjectAction, self).__init__(*args, **kwargs)
 
     def _validate(self):
-        self.action.valid = (
-            self._validate_domain_id() and
-            self._validate_parent_project() and
-            self._validate_project_absent())
+        self.action.valid = validate_steps([
+            self._validate_domain_id,
+            self._validate_keystone_user_parent_project,
+            self._validate_project_absent,
+        ])
         self.action.save()
 
     def _validate_domain_id(self):
@@ -53,7 +75,7 @@ class NewProjectAction(BaseAction, ProjectMixin, UserMixin):
 
         return super(NewProjectAction, self)._validate_domain_id()
 
-    def _validate_parent_project(self):
+    def _validate_keystone_user_parent_project(self):
         if self.parent_id:
             keystone_user = self.action.task.keystone_user
 
@@ -61,13 +83,13 @@ class NewProjectAction(BaseAction, ProjectMixin, UserMixin):
                 self.add_note(
                     'Parent id does not match keystone user project.')
                 return False
-            return super(NewProjectAction, self)._validate_parent_project()
+            return self._validate_parent_project()
         return True
 
-    def _pre_approve(self):
+    def _prepare(self):
         self._validate()
 
-    def _post_approve(self):
+    def _approve(self):
         project_id = self.get_cache('project_id')
         if project_id:
             self.action.task.cache['project_id'] = project_id
@@ -85,7 +107,7 @@ class NewProjectAction(BaseAction, ProjectMixin, UserMixin):
             self.action.task.cache['user_id'] = user_id
             self.add_note("User already given roles.")
         else:
-            default_roles = self.settings.get("default_roles", {})
+            default_roles = self.config.default_roles
 
             project_id = self.get_cache('project_id')
             keystone_user = self.action.task.keystone_user
@@ -105,14 +127,13 @@ class NewProjectAction(BaseAction, ProjectMixin, UserMixin):
             # put user_id into action cache:
             self.action.task.cache['user_id'] = user.id
             self.set_cache('user_id', user.id)
-            self.add_note(("Existing user '%s' attached to project %s" +
-                          " with roles: %s")
-                          % (user.name, project_id,
-                             default_roles))
+            self.add_note(
+                "Existing user '%s' attached to project %s with roles: %s"
+                % (user.name, project_id, default_roles))
 
     def _submit(self, token_data):
         """
-        Nothing to do here. Everything is done at post_approve.
+        Nothing to do here. Everything is done at the approve step.
         """
         pass
 
@@ -131,15 +152,29 @@ class NewProjectWithUserAction(UserNameAction, ProjectMixin, UserMixin):
         'email'
     ]
 
+    serializer = serializers.NewProjectWithUserSerializer
+
+    config_group = groups.DynamicNameConfigGroup(
+        children=[
+            fields.ListConfig(
+                "default_roles",
+                help_text="Roles to be given on project for the user.",
+                default=[],
+                sample_default=["member", "project_admin"]
+            ),
+        ],
+    )
+
     def __init__(self, *args, **kwargs):
         super(NewProjectWithUserAction, self).__init__(*args, **kwargs)
 
     def _validate(self):
-        self.action.valid = (
-            self._validate_domain_id() and
-            self._validate_parent_project() and
-            self._validate_project_absent() and
-            self._validate_user())
+        self.action.valid = validate_steps([
+            self._validate_domain_id,
+            self._validate_parent_project,
+            self._validate_project_absent,
+            self._validate_user,
+        ])
         self.action.save()
 
     def _validate_user(self):
@@ -147,29 +182,41 @@ class NewProjectWithUserAction(UserNameAction, ProjectMixin, UserMixin):
         user = id_manager.find_user(self.username, self.domain_id)
 
         if not user:
+            self.add_note(
+                "No user present with username '%s'. "
+                "Need to create new user." % self.username)
+            if not id_manager.can_edit_users:
+                self.add_note(
+                    'Identity backend does not support user editing, '
+                    'cannot create new user.')
+                return False
             # add to cache to use in template
             self.action.task.cache['user_state'] = "default"
             self.action.need_token = True
             self.set_token_fields(["password"])
-            self.add_note("No user present with username '%s'." %
-                          self.username)
             return True
 
-        if user.email != self.email:
+        if (not CONF.identity.username_is_email
+                and getattr(user, 'email', None) != self.email):
             self.add_note("Existing user '%s' with non-matching email." %
                           self.username)
             return False
 
         if not user.enabled:
+            self.add_note(
+                "Existing disabled user '%s' with matching email." %
+                self.email)
+            if not id_manager.can_edit_users:
+                self.add_note(
+                    'Identity backend does not support user editing, '
+                    'cannot renable user.')
+                return False
             self.action.state = "disabled"
             # add to cache to use in template
             self.action.task.cache['user_state'] = "disabled"
             self.action.need_token = True
             # as they are disabled we'll reset their password
             self.set_token_fields(["password"])
-            self.add_note(
-                "Existing disabled user '%s' with matching email." %
-                self.email)
             return True
         else:
             self.action.state = "existing"
@@ -198,10 +245,10 @@ class NewProjectWithUserAction(UserNameAction, ProjectMixin, UserMixin):
 
         self.action.save()
 
-    def _pre_approve(self):
+    def _prepare(self):
         self._validate()
 
-    def _post_approve(self):
+    def _approve(self):
         """
         Approving a new project means we set up the project itself,
         and if the user doesn't exist, create it right away. An existing
@@ -216,9 +263,9 @@ class NewProjectWithUserAction(UserNameAction, ProjectMixin, UserMixin):
             self.add_note("Project already created.")
         else:
             self.action.valid = (
-                self._validate_domain_id() and
-                self._validate_parent_project() and
-                self._validate_project_absent())
+                self._validate_domain_id()
+                and self._validate_parent_project()
+                and self._validate_project_absent())
             self.action.save()
 
             if not self.valid:
@@ -246,7 +293,7 @@ class NewProjectWithUserAction(UserNameAction, ProjectMixin, UserMixin):
 
     def _create_user_for_project(self):
         id_manager = user_store.IdentityManager()
-        default_roles = self.settings.get("default_roles", {})
+        default_roles = self.config.default_roles
 
         project_id = self.get_cache('project_id')
 
@@ -260,7 +307,7 @@ class NewProjectWithUserAction(UserNameAction, ProjectMixin, UserMixin):
                     user = id_manager.create_user(
                         name=self.username, password=password,
                         email=self.email, domain=self.domain_id,
-                        created_on=str(timezone.now()))
+                        created_on=str_datetime(timezone.now()))
                     self.set_cache('user_id', user.id)
                 else:
                     user = id_manager.get_user(user_id)
@@ -297,10 +344,9 @@ class NewProjectWithUserAction(UserNameAction, ProjectMixin, UserMixin):
                 raise
 
             self.set_cache('roles_granted', True)
-            self.add_note(("Existing user '%s' setup on project %s" +
-                          " with roles: %s")
-                          % (self.username, project_id,
-                             default_roles))
+            self.add_note(
+                "Existing user '%s' setup on project %s  with roles: %s"
+                % (self.username, project_id, default_roles))
         elif self.action.state == "disabled":
             user_id = self.get_cache('user_id')
             if not user_id:
@@ -344,17 +390,16 @@ class NewProjectWithUserAction(UserNameAction, ProjectMixin, UserMixin):
                     raise
                 self.set_cache('roles_granted', True)
 
-            self.add_note(("Existing user '%s' setup on project %s" +
-                          " with roles: %s")
-                          % (self.username, project_id,
-                             default_roles))
+            self.add_note(
+                "Existing user '%s' setup on project %s with roles: %s"
+                % (self.username, project_id, default_roles))
 
     def _submit(self, token_data):
         """
         The submit action is performed when a token is submitted.
         This is done to set a user password only, and so should now only
         change the user password. The project and user themselves are created
-        on post_approve.
+        on approve.
         """
 
         self._validate_user_submit()
@@ -399,10 +444,27 @@ class AddDefaultUsersToProjectAction(BaseAction, ProjectMixin, UserMixin):
         'domain_id',
     ]
 
+    serializer = serializers.AddDefaultUsersToProjectSerializer
+
+    config_group = groups.DynamicNameConfigGroup(
+        children=[
+            fields.ListConfig(
+                "default_users",
+                help_text="Users which this action should add to the project.",
+                default=[],
+            ),
+            fields.ListConfig(
+                "default_roles",
+                help_text="Roles which those users should get.",
+                default=[],
+            ),
+        ],
+    )
+
     def __init__(self, *args, **kwargs):
         super(AddDefaultUsersToProjectAction, self).__init__(*args, **kwargs)
-        self.users = self.settings.get('default_users', [])
-        self.roles = self.settings.get('default_roles', [])
+        self.users = self.config.default_users
+        self.roles = self.config.default_roles
 
     def _validate_users(self):
         id_manager = user_store.IdentityManager()
@@ -418,20 +480,22 @@ class AddDefaultUsersToProjectAction(BaseAction, ProjectMixin, UserMixin):
         return all_found
 
     def _pre_validate(self):
-        self.action.valid = self._validate_users()
+        self.action.valid = validate_steps([
+            self._validate_users,
+        ])
         self.action.save()
 
     def _validate(self):
-        self.action.valid = (
-            self._validate_users() and
-            self._validate_project_id()
-        )
+        self.action.valid = validate_steps([
+            self._validate_users,
+            self._validate_project_id,
+        ])
         self.action.save()
 
-    def _pre_approve(self):
+    def _prepare(self):
         self._pre_validate()
 
-    def _post_approve(self):
+    def _approve(self):
         id_manager = user_store.IdentityManager()
         self.project_id = self.action.task.cache.get('project_id', None)
         self._validate()
